package vods

import (
	"safetwitch-backend/extractor"
	"safetwitch-backend/extractor/twitch"
	"strconv"

	"github.com/gin-gonic/gin"
)

func Routes(route *gin.Engine) {
	vods := route.Group("/api/vods")

	vods.GET("/shelve/:streamerName", func(context *gin.Context) {
		data, err := twitch.GetStreamerVideoShelves(context.Param("streamerName"))
		if err != nil {
			context.Error(err)
			return
		}
		context.JSON(200, extractor.FormatMessage(data, true))
	})

	vods.GET("/:vodID", func(context *gin.Context) {
		data, err := twitch.GetVodMetadata(context.Param("vodID"))
		if err != nil {
			context.Error(err)
			return
		}

		data.Type = "vod"
		context.JSON(200, extractor.FormatMessage(data, true))
	})

	vods.GET("/comments/:vodID/:offset", func(context *gin.Context) {
		offset := context.Param("offset")
		o, err := strconv.Atoi(offset)
		if err != nil {
			context.Error(err)
			return
		}

		data, err := twitch.GetVODChat(context.Param("vodID"), o)
		if err != nil {
			context.Error(err)
			return
		}
		context.JSON(200, extractor.FormatMessage(data, true))
	})
}
